/*
 * Copyright 2010-2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License").
 * You may not use this file except in compliance with the License.
 * A copy of the License is located at
 *
 *  http://aws.amazon.com/apache2.0
 *
 * or in the "license" file accompanying this file. This file is distributed
 * on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */
package com.amazonaws.samples;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;

import com.amazonaws.AmazonClientException;
import com.amazonaws.ClientConfiguration;
import com.amazonaws.auth.AWSCredentials;
import com.amazonaws.regions.Region;
import com.amazonaws.regions.Regions;
import com.amazonaws.services.s3.AmazonS3Client;
import com.amazonaws.event.ProgressEvent;
import com.amazonaws.event.ProgressListener;
import com.amazonaws.services.s3.S3ClientOptions;
import com.amazonaws.services.s3.model.PutObjectRequest;
import com.amazonaws.services.s3.transfer.TransferManager;
import com.amazonaws.services.s3.transfer.Upload;

/**
 * Demonstrates how to upload data to Amazon S3, and track progress, using a
 * Swing progress bar.
 * <p>
 * <b>Prerequisites:</b> You must have a valid Amazon Web Services developer
 * account, and be signed up to use Amazon S3. For more information on Amazon
 * S3, see http://aws.amazon.com/s3.
 * <p>
 * Fill in your AWS access credentials in the provided credentials file
 * template, and be sure to move the file to the default location
 * (~/.aws/credentials) where the sample code will load the credentials from.
 * <p>
 * <b>WARNING:</b> To avoid accidental leakage of your credentials, DO NOT keep
 * the credentials file in your source directory.
 *
 * http://aws.amazon.com/security-credentials
 */
public class S3TransferProgressSample {

    private static AWSCredentials credentials = null;
    private static TransferManager tx;
    private static String bucketName;
    private static String prefix;

    private JProgressBar pb;
    private JFrame frame;
    private Upload upload;
    private JButton button;

    private static String env(String name, String def) {
        String v = System.getenv(name);
        return (v != null) ? v : def;
    }

    private static int envInt(String name, int def) {
        String v = System.getenv(name);
        return (v != null) ? Integer.valueOf(v) : def;
    }

    private static boolean envBoolean(String name, boolean def) {
        String v = System.getenv(name);
        return (v != null) ? Boolean.valueOf(v) : def;
    }

    // Example settings via env variables:
    // S3_ENDPOINT="http://backup42.docker.tx.caringo.com:4285" S3_ACCESS_KEY=a33898e802a42d5e099536b8a9727a3d S3_SECRET_KEY=secret mvn clean compile exec:java
    public static void main(String[] args) throws Exception {

        String endpoint = env("S3_ENDPOINT", "https://support.cloud.caringo.com"); // your domain
        bucketName = env("S3_BUCKET", "tmp"); // your domain
        prefix = env("S3_PREFIX", "output/uploader-swarm/S3TransferProgressSample/");
        // These uploader user credentials will expire periodically, please contact Caringo Support for latest.
        System.setProperty("aws.accessKeyId", env("S3_ACCESS_KEY", "6bbfc09e5f901061805c0e3fa77f18ff"));
        System.setProperty("aws.secretKey", env("S3_SECRET_KEY", "g9Dum60C1IjSGytKjh2kfSBnEC9YYInXo98fFCld"));

        int PART_SIZE_MB = envInt("S3_PART_SIZE", 2000 /* 2GB */);
        int CONNECTION_TIMEOUT_MS = envInt("S3_CONNECTION_TIMEOUT_MS", 240000);
        int SOCKET_TIMEOUT_MS = envInt("S3_SOCKET_TIMEOUT_MS", 240000);
        boolean enforceV4 = envBoolean("S3_ENFORCE_V4", true);
        boolean bucketInPath = envBoolean("S3_BUCKET_IN_PATH", true); // default is bucket in host, https://mybucket.support.cloud.caringo.com/object
        if (enforceV4) {
            System.setProperty("com.amazonaws.services.s3.enforceV4", "true"); // enforce V4 signature auth
        }
        ClientConfiguration cfg = new ClientConfiguration();
        System.out.println("Default getConnectionTimeout=" + cfg.getConnectionTimeout() + "ms, getSocketTimeout=" + cfg.getSocketTimeout() + "ms");
        cfg.setConnectionTimeout(CONNECTION_TIMEOUT_MS );
        cfg.setSocketTimeout(SOCKET_TIMEOUT_MS);
        S3ClientOptions s3opts = new S3ClientOptions();
        s3opts.setPathStyleAccess(bucketInPath);
        AmazonS3Client s3Client = new AmazonS3Client(cfg);
        s3Client.setS3ClientOptions(s3opts);
        if (enforceV4) {
            Region region = com.amazonaws.regions.Region.getRegion(Regions.US_EAST_1);
            s3Client.setRegion(region);
            s3Client.setSignerRegionOverride(region.getName()); // fixes "Invalid S3 URI" from AWS "s3.*" check
        }
        System.out.println("Copying " + endpoint + ", part size=" + PART_SIZE_MB + "MB, getConnectionTimeout=" + cfg.getConnectionTimeout() + "ms, getSocketTimeout=" + cfg.getSocketTimeout() + "ms, enforceV4="+enforceV4 + ", bucketInPath="+bucketInPath);
        s3Client.setEndpoint(endpoint);

        tx = new TransferManager(s3Client);

        new S3TransferProgressSample();
    }

    public S3TransferProgressSample() throws Exception {
        frame = new JFrame("Amazon S3 File Upload");
        button = new JButton("Choose File...");
        button.addActionListener(new ButtonListener());

        pb = new JProgressBar(0, 100);
        pb.setStringPainted(true);

        frame.setContentPane(createContentPane());
        frame.pack();
        frame.setVisible(true);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    }

    class ButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent ae) {
            JFileChooser fileChooser = new JFileChooser();
            int showOpenDialog = fileChooser.showOpenDialog(frame);
            if (showOpenDialog != JFileChooser.APPROVE_OPTION) return;

            createAmazonS3Bucket();

            ProgressListener progressListener = new ProgressListener() {
                public void progressChanged(ProgressEvent progressEvent) {
                    if (upload == null) return;

                    pb.setValue((int)upload.getProgress().getPercentTransferred());

                    switch (progressEvent.getEventCode()) {
                    case ProgressEvent.COMPLETED_EVENT_CODE:
                        pb.setValue(100);
                        break;
                    case ProgressEvent.FAILED_EVENT_CODE:
                        try {
                            AmazonClientException e = upload.waitForException();
                            JOptionPane.showMessageDialog(frame,
                                    "Unable to upload file to Amazon S3: " + e.getMessage(),
                                    "Error Uploading File", JOptionPane.ERROR_MESSAGE);
                        } catch (InterruptedException e) {}
                        break;
                    }
                }
            };

            File fileToUpload = fileChooser.getSelectedFile();
            PutObjectRequest request = new PutObjectRequest(
                    bucketName, prefix + fileToUpload.getName(), fileToUpload)
                .withGeneralProgressListener(progressListener);
            upload = tx.upload(request);
        }
    }

    private void createAmazonS3Bucket() {
        try {
            if (tx.getAmazonS3Client().doesBucketExist(bucketName) == false) {
                tx.getAmazonS3Client().createBucket(bucketName);
            }
        } catch (AmazonClientException ace) {
            JOptionPane.showMessageDialog(frame, "Unable to create a new Amazon S3 bucket: " + ace.getMessage(),
                    "Error Creating Bucket", JOptionPane.ERROR_MESSAGE);
        }
    }

    private JPanel createContentPane() {
        JPanel panel = new JPanel();
        panel.add(button);
        panel.add(pb);

        JPanel borderPanel = new JPanel();
        borderPanel.setLayout(new BorderLayout());
        borderPanel.add(panel, BorderLayout.NORTH);
        borderPanel.setBorder(BorderFactory.createEmptyBorder(20, 20, 20, 20));
        return borderPanel;
    }
}
